@@ -399,7 +399,11 @@ class Node(object):
             # incoming messages are handled by the action_handler instance
             # which is attached when the socket connection was made. wait()
             # is blocks forever (if no time is specified).
-            self.socketIO.wait()
+            try:
+                self.socketIO.wait()
+            except Exception as e:
+                self.log.critical('Listening thread exception')
+                self.log.debug(e)
 
     def __speaking_worker(self):
         """ Sending messages to central server.
@@ -413,54 +417,60 @@ class Node(object):
         self.log.debug("Waiting for results to send to the server")
 
         while True:
-            results = self.__docker.get_result()
+            try:
+                results = self.__docker.get_result()
 
-            # notify all of a crashed container
-            if results.status_code:
-                self.socketIO.emit(
-                    'container_failed',
-                    data={
-                        'node_id': self.server_io.whoami.id_,
-                        'status_code': results.status_code,
-                        'result_id': results.result_id,
-                        'collaboration_id': self.server_io.collaboration_id
-                    },
-                    namespace='/tasks'
+                # notify all of a crashed container
+                if results.status_code:
+                    self.socketIO.emit(
+                        'container_failed',
+                        data={
+                            'node_id': self.server_io.whoami.id_,
+                            'status_code': results.status_code,
+                            'result_id': results.result_id,
+                            'collaboration_id': self.server_io.collaboration_id
+                        },
+                        namespace='/tasks'
+                    )
+
+                self.log.info(
+                    f"Sending result (id={results.result_id}) to the server!")
+
+                # FIXME: why are we retrieving the result *again*? Shouldn't we
+                # just store the task_id when retrieving the task the first
+                # time?
+                response = self.server_io.request(
+                    f"result/{results.result_id}")
+                task_id = response.get("task").get("id")
+
+                if not task_id:
+                    self.log.error(
+                        f"task_id of result (id={results.result_id}) "
+                        f"could not be retrieved"
+                    )
+                    return
+
+                response = self.server_io.request(f"task/{task_id}")
+                initiator_id = response.get("initiator")
+
+                if not initiator_id:
+                    self.log.error(
+                        f"Initiator id from task (id={task_id})could not be "
+                        f"retrieved"
+                    )
+
+                self.server_io.patch_results(
+                    id=results.result_id,
+                    initiator_id=initiator_id,
+                    result={
+                        'result': results.data,
+                        'log': results.logs,
+                        'finished_at': datetime.datetime.now().isoformat(),
+                    }
                 )
-
-            self.log.info(
-                f"Sending result (id={results.result_id}) to the server!")
-
-            # FIXME: why are we retrieving the result *again*? Shouldn't we
-            # just store the task_id when retrieving the task the first time?
-            response = self.server_io.request(f"result/{results.result_id}")
-            task_id = response.get("task").get("id")
-
-            if not task_id:
-                self.log.error(
-                    f"task_id of result (id={results.result_id}) "
-                    f"could not be retrieved"
-                )
-                return
-
-            response = self.server_io.request(f"task/{task_id}")
-            initiator_id = response.get("initiator")
-
-            if not initiator_id:
-                self.log.error(
-                    f"Initiator id from task (id={task_id})could not be "
-                    f"retrieved"
-                )
-
-            self.server_io.patch_results(
-                id=results.result_id,
-                initiator_id=initiator_id,
-                result={
-                    'result': results.data,
-                    'log': results.logs,
-                    'finished_at': datetime.datetime.now().isoformat(),
-                }
-            )
+            except Exception as e:
+                self.log.critical('Spreaker thread exception')
+                self.log.debug(e)
 
     def authenticate(self):
         """ Authenticate to the central server
@@ -471,7 +481,9 @@ class Node(object):
         api_key = self.config.get("api_key")
 
         keep_trying = True
-        while keep_trying:
+        i = 0
+        while keep_trying and i < 10:
+            i = i + 1
             try:
                 self.server_io.authenticate(api_key)
 
@@ -533,22 +545,22 @@ class Node(object):
         """
 
         self.socketIO = SocketIO(request_timeout=60)
-        self.socketIO.connect(
-            url=f'{self.server_io.host}:{self.server_io.port}',
-            headers=self.server_io.headers,
-            namespaces=['/tasks']
-        )
 
         self.socketIO.register_namespace(NodeTaskNamespace('/tasks'))
         NodeTaskNamespace.node_worker_ref = self
 
+        self.socketIO.connect(
+            url=f'{self.server_io.host}:{self.server_io.port}',
+            headers=self.server_io.headers,
+        )
+
         # Log the outcome
         while not self.socketIO.connected:
-            self.log.debug('waiting for socket connection...')
+            self.log.debug('Waiting for socket connection...')
             time.sleep(1)
 
-        self.log.info(f'connected to host={self.server_io.host} '
-                      f'on port={self.server_io.port}')
+        self.log.info(f'Connected to host={self.server_io.host} on port='
+                      f'{self.server_io.port}')
 
     def get_task_and_add_to_queue(self, task_id):
         """Fetches (open) task with task_id from the server.
